import numpy as np
from qpython import qconnection
from .advancedqreader import AdvancedQReader


class Server:
    def __init__(self):
        self.name = ""
        self.host = ""
        self.port = 0
        self.username = ""
        self.password = ""
        self.pandas = False

    def to_string(self):
        return self.name


class KDB:
    def __init__(self, server):
        self.ds = server.name
        self.q = qconnection.QConnection(host=server.host, port=server.port, username=server.username,
                                         password=server.password,
                                         pandas=server.pandas,
                                         reader_class=AdvancedQReader)

    def get_conn(self):
        return self.q

    def close_conn(self):
        self.q.close()

    def query_sync(self, query, *parameters, **options):
        self.q.open()
        try:
            return self.q.sendSync(query, *parameters, **options)
        except Exception as e:
            print("DB[{}] error: {}".format(self.ds, e.args[0]))
            print("query: {}".format(query))
            print("parameters: {}".format(parameters))
            print("options: {}".format(options))
        finally:
            self.close_conn()

    def to_sym(self, s):
        return np.string_(s, encoding='utf-8')

    def to_sym_list(self, l):
        """
        convert np.array[object] to np.array[np.string_ with utf-8]
        :param l: df['col'].values
        :return: np.string_ with utf-8
        """
        return np.char.encode(l.astype(np.unicode_), encoding='utf-8')

    def to_date(self, l):
        return l.astype('M8[D]')
